# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Authors: Amador Pahim <apahim@redhat.com>

"""
Plugin to run Golang tests in Avocado
"""

import fnmatch
import glob
import os
import re

from avocado.core.nrunner.runnable import Runnable
from avocado.core.plugin_interfaces import Resolver
from avocado.core.resolver import ReferenceResolution, ReferenceResolutionResult
from avocado.utils import path as utils_path

try:
    GO_BIN = utils_path.find_command("go")
except utils_path.CmdNotFoundError:
    GO_BIN = None


TEST_RE = re.compile(r"^func\s(Test|Example)[A-Z]")


def find_tests(test_path):
    test_suite = []
    with open(test_path, "r", encoding="utf-8") as test_file_fd:
        for line in test_file_fd.readlines():
            if TEST_RE.match(line):
                test_suite.append(line.split()[1].split("(")[0])

    return test_suite


def find_files(path, recursive=True):
    pattern = "*_test.go"
    if recursive:
        matches = []
        for root, _, filenames in os.walk(path):
            for filename in fnmatch.filter(filenames, pattern):
                matches.append(os.path.join(root, filename))
        return matches

    if path != os.path.curdir:
        pattern = os.path.join(path, pattern)
    return glob.iglob(pattern)


class GolangResolver(Resolver):

    name = "golang"
    description = "Test resolver for Go language tests"

    @staticmethod
    def resolve(reference):  # pylint: disable=W0221

        if GO_BIN is None:
            return ReferenceResolution(
                reference,
                ReferenceResolutionResult.NOTFOUND,
                info="go binary not found",
            )

        package_paths = []
        test_files = []
        go_path = os.environ.get("GOPATH")
        if go_path is not None:
            for directory in go_path.split(os.pathsep):
                pkg_path = os.path.join(os.path.expanduser(directory), "src")
                package_paths.append(pkg_path)

        for package_path in package_paths:
            url_path = os.path.join(package_path, reference)
            files = find_files(url_path)
            if files:
                test_files.append((package_path, files))
                break

        runnables = []
        for package_path, test_files_list in test_files:
            for test_file in test_files_list:
                for item in find_tests(test_file):
                    common_prefix = os.path.commonprefix([package_path, test_file])
                    match_package = os.path.relpath(test_file, common_prefix)
                    test_name = f"{os.path.dirname(match_package)}:{item}"
                    runnables.append(Runnable("golang", uri=test_name))

        if runnables:
            return ReferenceResolution(
                reference, ReferenceResolutionResult.SUCCESS, runnables
            )

        return ReferenceResolution(reference, ReferenceResolutionResult.NOTFOUND)



import subprocess

from avocado_golang.golang import GO_BIN

from avocado.core.nrunner.app import BaseRunnerApp
from avocado.core.nrunner.runner import BaseRunner
from avocado.core.utils import messages


class GolangRunner(BaseRunner):
    """Runner for Golang tests.

    When creating the Runnable, use the following attributes:

     * kind: should be 'golang';

     * uri: module name and optionally a test method name, separated by colon;

     * args: not used

     * kwargs: not used

    Example:

       runnable = Runnable(kind='golang',
                           uri='countavocados:ExampleContainers')
    """

    name = "golang"
    description = "Runner for Golang tests"

    def run(self, runnable):
        # pylint: disable=W0201
        self.runnable = runnable
        error_msgs = []
        if not GO_BIN:
            error_msgs.append('"go" binary is not available')

        if not self.runnable.uri:
            error_msgs.append("an empty URI was given")

        if error_msgs:
            yield self.prepare_status(
                "finished", {"result": "error", "output": "and , ".join(error_msgs)}
            )
            return

        yield messages.StartedMessage.get()
        module_test = self.runnable.uri.split(":", 1)
        module = module_test[0]
        try:
            test = module_test[1]
        except IndexError:
            test = None

        cmd = [GO_BIN, "test", "-v", module]
        if test is not None:
            cmd += ["-run", f"{test}$"]

        process = subprocess.Popen(
            cmd,
            stdin=subprocess.DEVNULL,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        def poll_proc():
            return process.poll() is not None

        yield self.prepare_status("started")
        yield from self.running_loop(poll_proc)

        result = "pass" if process.returncode == 0 else "fail"
        yield messages.StdoutMessage.get(process.stdout.read())
        yield messages.StderrMessage.get(process.stderr.read())
        yield messages.FinishedMessage.get(result, returncode=process.returncode)


class RunnerApp(BaseRunnerApp):
    PROG_NAME = "avocado-runner-golang"
    PROG_DESCRIPTION = "nrunner application for golang tests"
    RUNNABLE_KINDS_CAPABLE = ["golang"]


def main():
    app = RunnerApp(print)
    app.run()


if __name__ == "__main__":
    main()



#!/bin/env python3
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Amador Pahim <apahim@redhat.com>

from setuptools import setup

# Handle systems with setuptools < 40
try:
    from setuptools import find_namespace_packages
except ImportError:
    packages = ["avocado_golang"]
else:
    packages = find_namespace_packages(include=["avocado_golang"])

VERSION = open("VERSION", "r", encoding="utf-8").read().strip()

setup(
    name="avocado-framework-plugin-golang",
    description="Avocado Plugin for Execution of Golang tests",
    version=VERSION,
    author="Avocado Developers",
    author_email="avocado-devel@redhat.com",
    url="http://avocado-framework.github.io/",
    packages=packages,
    include_package_data=True,
    install_requires=[f"avocado-framework=={VERSION}"],
    test_suite="tests",
    entry_points={
        "console_scripts": [
            "avocado-runner-golang = avocado_golang.runner:main",
        ],
        "avocado.plugins.resolver": [
            "golang = avocado_golang.golang:GolangResolver",
        ],
        "avocado.plugins.runnable.runner": [
            "golang = avocado_golang.runner:GolangRunner"
        ],
    },
)



import os
import unittest.mock

import avocado_golang.golang

from avocado.core.resolver import ReferenceResolutionResult

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


@unittest.skipIf(avocado_golang.golang.GO_BIN is None, "go binary not found")
class ResolverModule(unittest.TestCase):
    """
    Test golang resolution when a module name is given
    """

    def setUp(self):
        self.previous_go_path = os.environ.get("GOPATH", None)
        os.environ["GOPATH"] = THIS_DIR

    def test_resolver_no_go_bin(self):
        with unittest.mock.patch("avocado_golang.golang.GO_BIN", None):
            res = avocado_golang.golang.GolangResolver().resolve("countavocados")
        self.assertEqual(res.reference, "countavocados")
        self.assertEqual(res.result, ReferenceResolutionResult.NOTFOUND)

    def test_resolver(self):
        res = avocado_golang.golang.GolangResolver().resolve("countavocados")
        self.assertEqual(res.result, ReferenceResolutionResult.SUCCESS)
        self.assertEqual(len(res.resolutions), 3)
        empty_container = res.resolutions[0]
        self.assertEqual(empty_container.kind, "golang")
        self.assertEqual(empty_container.uri, "countavocados:TestEmptyContainers")
        no_container = res.resolutions[1]
        self.assertEqual(no_container.kind, "golang")
        self.assertEqual(no_container.uri, "countavocados:TestNoContainers")
        example = res.resolutions[2]
        self.assertEqual(example.kind, "golang")
        self.assertEqual(example.uri, "countavocados:ExampleContainers")

    def tearDown(self):
        if self.previous_go_path is not None:
            os.environ["GOPATH"] = self.previous_go_path
